#ifndef OGREFRAMEWORK_H
#define OGREFRAMEWORK_H

#include <OGRE/Ogre.h>
#include <OIS/OIS.h>

#ifdef GAME_STATIC_BUILD
    #include <OGRE/RenderSystems/GL/OgreGLPlugin.h>
    #include <OGRE/Plugins/CgProgramManager/OgreCgPlugin.h>
    #include <OGRE/Plugins/ParticleFX/OgreParticleFXPlugin.h>
    #include <OgreOggSound/OgreOggSoundPlugin.h>
#endif

class OgreFramework : public Ogre::Singleton<OgreFramework>, OIS::KeyListener, OIS::MouseListener
{
public:
    OgreFramework();
    ~OgreFramework();

    bool initOgre(Ogre::String windowTitle, OIS::KeyListener *keyListener = 0, OIS::MouseListener *mouseListener = 0);
    void updateOgre(double timeSinceLastFrame) {}

    bool keyPressed(const OIS::KeyEvent &keyEventRef) { return true; }
    bool keyReleased(const OIS::KeyEvent &keyEventRef) { return true; }

    bool mouseMoved(const OIS::MouseEvent &event) { return true; }
    bool mousePressed(const OIS::MouseEvent &event, OIS::MouseButtonID id) { return true; }
    bool mouseReleased(const OIS::MouseEvent &event, OIS::MouseButtonID id) { return true; }

    Ogre::Root *m_root;
    Ogre::RenderWindow *m_renderWindow;
    Ogre::Viewport *m_viewport;
    Ogre::Timer *m_timer;

    OIS::InputManager *m_inputManager;
    OIS::Keyboard *m_keyboard;
    OIS::Mouse *m_mouse;

private:
    OgreFramework(const OgreFramework&);
    OgreFramework& operator= (const OgreFramework&);

    void installPlugins();
    void loadResources();

#ifdef GAME_STATIC_BUILD
    Ogre::GLPlugin *m_glPlugin;
    Ogre::CgPlugin *m_cgPlugin;
    Ogre::ParticleFXPlugin *m_particleFXPlugin;
    OgreOggSound::OgreOggSoundPlugin *m_soundPlugin;
#endif

};

#endif  // OGREFRAMEWORK_H
