/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Compositor listener for motion blur effect. It also provides
 *  a way to dynamically change blur parameters during runtime.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef BLURLISTENER_H
#define BLURLISTENER_H

#include <OGRE/Ogre.h>
#include <OGRE/OgreCompositorInstance.h>

/// Motion blur compositor listener
class BlurListener : public Ogre::CompositorInstance::Listener
{
public:
    BlurListener() { m_sampleStrength = 0; }
    ~BlurListener() {}

    /** Update blur parameters. */
    void notifyMaterialRender(Ogre::uint32 pass_id, Ogre::MaterialPtr &mat)
    {
        Ogre::GpuProgramParametersSharedPtr blurParams =
            mat->getBestTechnique()->getPass(0)->getFragmentProgramParameters();
        //blurParams->setNamedConstant("sampleDist", Ogre::Real(1.0));
        blurParams->setNamedConstant("sampleStrength", m_sampleStrength);
    }

    /** Set new blur strength value. */
    void setSampleStrength(Ogre::Real sampleStrength) { m_sampleStrength = sampleStrength; }

private:
    Ogre::Real m_sampleStrength;
};

#endif  // BLURLISTENER_H
