/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Manages all in-game cameras.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef CAMERA_MANAGER_H
#define CAMERA_MANAGER_H

#include "dynamiccamera.h"
#include "staticcamera.h"
#include "sidecamera.h"

/** We have different camera types. */
enum CameraType
{
    CAMERA_GENERIC,
    CAMERA_STATIC,
    CAMERA_DYNAMIC,
    CAMERA_SIDE,
};

/** Camera associated with a player. */
struct PlayerCamera
{
    GameCamera *camera;
    int type;
    int player;
};

/** Manager of all cameras in the game. */
class CameraManager
{
public:
    /** CameraManager needs access to the scene. */
    CameraManager(Ogre::SceneManager *manager);

    /** Destroy all cameras. */
    ~CameraManager();

    /** Create camera of certain type and aspect ratio associated with certain player. */
    GameCamera *createCamera(int type, Ogre::Real aspect, int player);

    /** Return first found camera of given type belonging to the player. */
    GameCamera *getCamera(int type, int player);

    /** Return n-th camera of given type belonging to the player. Counting from 0. */
    GameCamera *getCamera(int type, int player, int n);

    /** Update position and orientation of all cameras of given player. */
    void update(double timeSinceLastFrame, const Ogre::SceneNode *target, int player);

private:
    Ogre::SceneManager *m_sceneManager;
    std::vector<PlayerCamera> m_cameras;
};

#endif  // CAMERA_MANAGER_H
