/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Camera that dynamically follows the ship.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DYNAMIC_CAMERA_H
#define DYNAMIC_CAMERA_H

#include "gamecamera.h"

class DynamicCamera : public GameCamera
{
public:
    /** Reset camera and set new position. */
    void setPosition(const Ogre::Vector3 &pos);

    /** Update position and orientation of the camera. */
    void update(double timeSinceLastFrame, const Ogre::SceneNode *target);

protected:
    /** Update position of the camera. */
    void updatePos(double timeSinceLastFrame, const Ogre::SceneNode *target);

    /** Update orientation of the camera. */
    void updateLook(double timeSinceLastFrame, const Ogre::SceneNode *target);

private:
    Ogre::Vector3 m_desiredPos; /**< Position of camera in idle state. */
    Ogre::Vector3 m_desiredLook;/**< Desired look at position of camera. */
    Ogre::Vector3 m_realPos;    /**< Actual position (equals desired if idle) */
    Ogre::Vector3 m_realLook;   /**< Where is the camera actually looking at. */
};

#endif  // DYNAMIC_CAMERA_H
