/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Main class of the game.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "gameapp.h"
#include "gamestate.h"
#include "menustate.h"

// Visual Leak Detector
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#include "vld.h"
#endif

// At the beginning, we have no instance
GameApp *GameApp::m_instance = NULL;

GameApp::GameApp()
{
    m_appStateManager = NULL;
}

GameApp::~GameApp()
{
    delete m_appStateManager;
    delete OgreFramework::getSingletonPtr();
}

GameApp *GameApp::getSingleton()
{
    if (m_instance == NULL)
        m_instance = new GameApp;

    return m_instance;
}

void GameApp::start()
{
    new OgreFramework;

    if (!OgreFramework::getSingletonPtr()->initOgre("Velocity Overflow", NULL, NULL))
        return;

    m_appStateManager = new AppStateManager;

    MenuState::create(m_appStateManager, "MenuState");
    GameState::create(m_appStateManager, "GameState");

    m_appStateManager->start(m_appStateManager->findByName("MenuState"));
}
