/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Camera wrapper is used for updating position and orientation of a camera.
 *  This basic implementation is just simple camera, but it is supposed to be
 *  reimplemented to include more features. The actual camera (Ogre::Camera object)
 *  is not created inside this class, but it should be created and destroyed
 *  outside using Ogre::SceneManager.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "gamecamera.h"

void GameCamera::setPosition(const Ogre::Vector3 &pos)
{
    m_camera->setPosition(pos);
}

void GameCamera::update(double timeSinceLastFrame, const Ogre::SceneNode *target)
{
    updatePos(timeSinceLastFrame, target);
    updateLook(timeSinceLastFrame, target);
}

void GameCamera::updatePos(double timeSinceLastFrame, const Ogre::SceneNode *target)
{
    m_camera->setPosition(target->getPosition());
}

void GameCamera::updateLook(double timeSinceLastFrame, const Ogre::SceneNode *target)
{
    m_camera->lookAt(target->getPosition());
}
