/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Camera wrapper is used for updating position and orientation of a camera.
 *  This basic implementation is just simple camera, but it is supposed to be
 *  reimplemented to include more features. The actual camera (Ogre::Camera object)
 *  is not created inside this class, but it should be created and destroyed
 *  outside using Ogre::SceneManager.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef GAMECAMERA_H
#define GAMECAMERA_H

#include <OGRE/Ogre.h>

class GameCamera
{
public:
    GameCamera() : m_camera(0) {}

    /** Pointer to camera object. */
    Ogre::Camera *getCamera() { return m_camera; }

    /** Set camera object. */
    void setCamera(Ogre::Camera *camera) { m_camera = camera; }

    /** Set camera to new position. */
    virtual void setPosition(const Ogre::Vector3 &pos);

    /** Update position and orientation of the camera. */
    virtual void update(double timeSinceLastFrame, const Ogre::SceneNode *target);

protected:
    /** Update position of the camera. */
    virtual void updatePos(double timeSinceLastFrame, const Ogre::SceneNode *target);

    /** Update orientation of the camera. */
    virtual void updateLook(double timeSinceLastFrame, const Ogre::SceneNode *target);

protected:
    Ogre::Camera *m_camera;
};

#endif  // GAMECAMERA_H
