/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Display screen with GUI wherever it is needed.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "gui.h"

#include "OgreSDK/FileSystemLayerImpl.h"

#ifndef MYGUI_LOG_FILE
#define MYGUI_LOG_FILE "mygui.log"
#endif

// At the beginning, we have no instance
Gui *Gui::m_instance = NULL;

Gui::Gui()
{
    // Platform independent path to user's files
    OgreBites::FileSystemLayerImpl fs("velocityoverflow");

    m_platform = new MyGUI::OgrePlatform;
    m_gui = new MyGUI::Gui;

    m_platform->initialise(OgreFramework::getSingletonPtr()->m_renderWindow,
                           NULL, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
                           fs.getWritablePath(MYGUI_LOG_FILE));
    m_gui->initialise();
}

Gui::~Gui()
{
    m_gui->shutdown();
    m_platform->shutdown();

    delete m_gui;
    delete m_platform;
}

Gui *Gui::getSingleton()
{
    if (m_instance == NULL)
        m_instance = new Gui;

    return m_instance;
}

void Gui::attach(Ogre::SceneManager *manager)
{
    m_platform->getRenderManagerPtr()->setSceneManager(manager);
}

void Gui::detach()
{
    m_platform->getRenderManagerPtr()->setSceneManager(NULL);
}

void Gui::load(const std::string &layout)
{
    // Unload current layout
    MyGUI::LayoutManager::getInstance().unloadLayout(m_widgets);

    // Load layout from file and resize the GUI to fit the view
    m_widgets = MyGUI::LayoutManager::getInstance().loadLayout(layout);
    MyGUI::LayerManager::getInstance().resizeView(MyGUI::RenderManager::getInstance().getViewSize());
}
