/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Display screen with GUI wherever it is needed.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef GUI_H
#define GUI_H

#include "OgreFramework/ogreframework.h"

#include <MyGUI.h>
#include <MyGUI_OgrePlatform.h>

/** Simple wrapper for GUI management. Before using any
  * of its methods, attach() must be called first! */
class Gui
{
protected:
    Gui();
    ~Gui();

public:
    /** Get pointer to the actual object. */
    static Gui *getSingleton();

    /** Attach the GUI to a scene. */
    void attach(Ogre::SceneManager *manager);

    /** Detach the GUI from current scene. */
    void detach();

    /** Load new layout. */
    void load(const std::string &layout);

    /** Return pointer to the Gui object. */
    MyGUI::Gui *getGui() { return m_gui; }

    /** Return pointer to the OgrePlatform object. */
    MyGUI::OgrePlatform *getPlatform() { return m_platform; }

private:
    /** The only instance of this class. */
    static Gui *m_instance;

    MyGUI::Gui* m_gui;
    MyGUI::OgrePlatform *m_platform;
    MyGUI::VectorWidgetPtr m_widgets;
};

#endif  // GUI_H
