/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Helper structures for Level class. Most of them are used to describe differend kinds of 3D objects
 *  which should interact with the player (e.g. checkpoints). Also used by other classes.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef LEVELOBJECTS_H
#define LEVELOBJECTS_H

#include "BtOgre/BtOgrePG.h"
#include "BtOgre/BtOgreGP.h"
#include "BtOgre/BtOgreExtras.h"

#include <OGRE/Ogre.h>

// We need to distinguish between multiple object types during collision.
// Each object type has its own range of IDs. First we tag an object with
// ID and later in Physics class we determine the type by these ranges.
const unsigned int checkpointMinId = 0;
const unsigned int checkpointMaxId = 999;
const unsigned int acceleratorMinId = 1000;
const unsigned int acceleratorMaxId = 1999;
const unsigned int slowdownMinId = 2000;
const unsigned int slowdownMaxId = 2999;

/** Location represents object's position and rotation in space. */
struct Location
{
    Ogre::Vector3 position;
    Ogre::Quaternion rotation;
};

/** Checkpoint on the track. */
struct Checkpoint : public Location
{
    Ogre::SceneNode *node;          /**< Node is used for creating motion state. */
    BtOgre::RigidBodyState *state;  /**< Motion state. */
    btRigidBody *body;              /**< Static rigid body. */
    unsigned int id;                /**< Unique ID so we can identify checkpoints in physical world. */
};

/** Accelerator (or Slowdown) object on the track. */
struct Accelerator : public Checkpoint
{
    Ogre::Entity *entity;           /**< Trigger entity is used to create collision shape. */
    btCollisionShape *shape;        /**< Collision shape. */
};

#endif  // LEVELOBJECTS_H
