/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Initial screen of the game.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef MENUSTATE_H
#define MENUSTATE_H

#include "OgreFramework/appstate.h"
#include "ship.h"
#include "gui.h"

/** This state is active while in menu. */
class MenuState : public AppState
{
public:
    MenuState();
    ~MenuState();

    DECLARE_APPSTATE_CLASS(MenuState)

    void enter();
    void exit();

    bool keyPressed(const OIS::KeyEvent &arg);
    bool keyReleased(const OIS::KeyEvent &arg);

    bool mouseMoved(const OIS::MouseEvent &arg);
    bool mousePressed(const OIS::MouseEvent &arg, OIS::MouseButtonID id);
    bool mouseReleased(const OIS::MouseEvent &arg, OIS::MouseButtonID id);

    void guiSinglePressed(MyGUI::Widget *);
    void guiSplitPressed(MyGUI::Widget *);
    void guiScorePressed(MyGUI::Widget *);
    void guiOptionsPressed(MyGUI::Widget *);
    void guiCreditsPressed(MyGUI::Widget *);
    void guiExitPressed(MyGUI::Widget *);

    void guiLeftShipPressed(MyGUI::Widget *);
    void guiRightShipPressed(MyGUI::Widget *);
    void guiSavePressed(MyGUI::Widget *);
    void guiOkPressed(MyGUI::Widget *);
    void guiCancelPressed(MyGUI::Widget *);

    /** This is handler for every checkbox. It is used only for toggling
      * the checkboxes. It is needed because of the way how MyGUI works. */
    void guiCheckBoxPressed(MyGUI::Widget *widget);

    /** Update is called automatically every frame. */
    void update(double timeSinceLastFrame);

    /** Place objects into the scene. */
    void createScene();

    /** Load and show GUI screen. */
    void showGui();

private:
    /**< Scene manager for the menu scene. */
    Ogre::SceneManager *m_sceneManager;
    
    /** Current state of the FSM, which is used to switch menu screens. */
    int m_currentState;

    Ship *m_ship1;                      /**< Displayed ship on the left. */
    Ship *m_ship2;                      /**< Displayed ship on the right. */
    Ogre::MaterialPtr m_ship1Mat;       /**< Pointer to material of the left ship. */
    Ogre::MaterialPtr m_ship2Mat;       /**< Pointer to material of the right ship. */
    Ogre::ColourValue m_ship1Specular;  /**< Original specular color of the left ship. */
    Ogre::ColourValue m_ship2Specular;  /**< Original specular color of the right ship. */

    bool m_quit;                        /**< Quit the game? */
};

#endif  // MENUSTATE_H
