/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  This class connects the game with the Bullet physics engine.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef PHYSICS_H
#define PHYSICS_H

#include "BtOgre/BtOgrePG.h"
#include "BtOgre/BtOgreGP.h"
#include "BtOgre/BtOgreExtras.h"

#include "player.h"
#include "ship.h"
#include "trackraycallback.h"

/** This is called by Bullet 60 times per second just before the
  * physics simulation takes step. */
static void preTickCallback(btDynamicsWorld *world, btScalar timeStep);

/** Connection between our game and physics engine. */
class Physics
{
protected:
    Physics();
    ~Physics();

public:
    /** Get pointer to the actual object. */
    static Physics *getSingleton();
    
    /** Reset object to its initial state. */
    void destroy() { delete this; }

    /** Get pointer to the physics world. */
    btDynamicsWorld *getWorld() { return m_world; }

    /** Add ship to the physics world. */
    void addShip(Ship *ship, Player *player);

    /** Get pointer to the debug drawer. */
    BtOgre::DebugDrawer *getDbgdraw() { return m_dbgdraw; }

    /** Enable/Disable physics debugging visualizations. */
    void toggleDebug();

    /** Return if we are in debug mode or not. */
    bool isDebug() { return m_debug; }

    /** Do a ray casting and calculate first hit. */
    bool rayCast(const btVector3 &start, const btVector3 &end, btVector3 &hit);

    /** This method is called by static function preTickCallback(). */
    void processCallback(btScalar timeStep);

private:
    static Physics *m_instance;         /**< The only instance of this class. */
    
    btDynamicsWorld *m_world;
    BtOgre::DebugDrawer *m_dbgdraw;
    bool m_debug;

    btAxisSweep3 *m_broadphase;
    btDefaultCollisionConfiguration *m_collisionConfig;
    btCollisionDispatcher *m_dispatcher;
    btSequentialImpulseConstraintSolver *m_solver;

    std::vector<Ship *> m_ships;        /**< List of ships in the race. */
};

#endif  // PHYSICS_H
