/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Class that represents player in the game.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "player.h"

Player::Player()
{
    init();
}

Player::~Player()
{
}

void Player::init()
{
    m_ship = NULL;

    m_wrongWay = false;
    m_currentCheckpoint = 0;
    m_nextCheckpoint = 0;

    m_firstCheckpoint = 0;
    m_halfCheckpoint = 0;
    m_lastHalf = 0;
    m_laps = 0;

    m_totalTime = 0;

    m_state = PLAYER_STATE_WAITING;
}

unsigned int Player::getCurrentCheckpoint() const
{
    return m_currentCheckpoint;
}

void Player::setCurrentCheckpoint(unsigned int cp)
{
    if (m_currentCheckpoint == cp)
        return;

    // Check if we have traveled half of the track
    if (cp == m_firstCheckpoint && m_lastHalf == m_halfCheckpoint) {
        m_lastHalf = cp;
        m_laps++;
    } else if (cp == m_halfCheckpoint && m_lastHalf == m_firstCheckpoint) {
        m_lastHalf = cp;
        m_laps++;
    }

    m_currentCheckpoint = cp;
}

unsigned int Player::getLaps() const
{
    // If we are at the beginning, return 1 lap
    if (!m_laps)
        return 1;

    // Otherwise compute number from half laps value
    return (m_laps / 2 + 1);
}

void Player::setLaps(unsigned int laps)
{
    m_laps = laps;
}
