/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Timer that is used to measure time during the race.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "racetimer.h"

#include <OGRE/OgreStringConverter.h>

RaceTimer::RaceTimer()
{
    m_timer = new Ogre::Timer;
    m_pausedMs = 0;
    m_paused = false;
}

RaceTimer::~RaceTimer()
{
    delete m_timer;
}

void RaceTimer::start()
{
    m_timer->reset();
    m_pausedMs = 0;
    m_paused = false;
}

void RaceTimer::pause()
{
    if (!m_paused) {
        m_pauseTime = m_timer->getMilliseconds();
        m_paused = true;
    }
}

void RaceTimer::resume()
{
    if (m_paused) {
        m_pausedMs += m_timer->getMilliseconds() - m_pauseTime;
        m_paused = false;
    }
}

unsigned long RaceTimer::getMilliseconds() const
{
    return (m_timer->getMilliseconds() - m_pausedMs);
}

const std::string RaceTimer::timeToString(unsigned long ms, bool longer)
{
    
    std::string msPart;     // Milliseconds
    std::string secPart;    // Seconds
    std::string minPart;    // Minutes

    secPart = Ogre::StringConverter::toString((ms / 1000) % 60);
    if (secPart.length() == 1)
        secPart = "0" + secPart;

    minPart = Ogre::StringConverter::toString((ms / 1000 / 60) % 60);

    if (longer) {
        msPart = Ogre::StringConverter::toString((ms % 1000) / 10);
        if (msPart.length() == 1)
            msPart = "0" + msPart;

        if (minPart.length() == 1)
            minPart = "0" + minPart;
    }
    else {
        msPart = Ogre::StringConverter::toString((ms % 1000) / 100);
    }

    // Final time
    std::string time = minPart + ":" + secPart + ":" + msPart;
    return time;
}
