/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Timer that is used to measure time during the race.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef RACETIMER_H
#define RACETIMER_H

#include <OGRE/OgreTimer.h>

class RaceTimer
{
public:
    RaceTimer();
    ~RaceTimer();

    /** (Re)start the timer. */
    void start();

    /** Pause the race time. */
    void pause();

    /** Resume the race time. */
    void resume();

    /** Get time of race in milliseconds. */
    unsigned long getMilliseconds() const;

    /** Convert time (in ms) to readable string format (0:00:0 or 00:00:00). */
    static const std::string timeToString(unsigned long ms, bool longer=false);

private:
    Ogre::Timer *m_timer;       /**< Timer used to measure time. */
    unsigned int m_pauseTime;   /**< Value of timer at the moment when pause() is called. */
    unsigned int m_pausedMs;    /**< How long was the race paused totally. */
    bool m_paused;              /**< Is the race paused? */
};

#endif  // RACETIMER_H
