/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Manages and plays all sounds and music.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef SOUNDMANAGER_H
#define SOUNDMANAGER_H

#include <OgreOggSound/OgreOggSound.h>

/** Management of sounds and music. */
class SoundManager
{
protected:
    SoundManager();
    ~SoundManager();

public:
    /** Get pointer to the actual object. */
    static SoundManager *getSingleton();

    /** Attach the sound manage to a scene. */
    void attach(Ogre::SceneManager *sceneManager);

    /** Detach the sound manager from current scene and free resources. */
    void detach();

    /** Load sounds that will play in menu. */
    void createMenuSounds();

    /** Load sounds that will play during gameplay. */
    void createGameSounds();

    /** Mute/unmute all sounds immediately. */
    void setMute(bool muted);

    /** Has the current song in gameplay stopped playing? */
    bool isSongStopped() const;

    /** Play a specific sound. */
    void play(const std::string &name);

    /** Play random song. */
    void playRandom();

private:
    /**< The only instance of this class. */
    static SoundManager *m_instance;
    
    OgreOggSound::OgreOggSoundManager *m_manager;

    bool m_muted;           /**< Pause/resume all sounds. */
    std::string m_song;     /**< Name of currently played song in gameplay. */
};

#endif  // SOUNDMANAGER_H
