/**
 *  @author Marian Ondrejicek (xondre02)
 *  @date   May 2012
 *
 *  Class TrackRayCallback overrides default ray test collision check.
 *  This way we can check if the ray hits only specific object (track)
 *  and filter out other objects.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef TRACKRAYCALLBACK_H
#define TRACKRAYCALLBACK_H

#include <bullet/BulletCollision/CollisionDispatch/btCollisionWorld.h>

/** Class used to override default ray test method so we can test only specific objects. */
class TrackRayCallback : public btCollisionWorld::ClosestRayResultCallback
{
public:
    TrackRayCallback(const btVector3 & rayFrom,const btVector3 & rayTo) :
      btCollisionWorld::ClosestRayResultCallback(rayFrom, rayTo) {}

    ~TrackRayCallback() {}
    
    /** Override default ray test method. */
    bool needsCollision(btBroadphaseProxy* proxy0) const
    {
        // Based on: http://bulletphysics.org/Bullet/phpBB3/viewtopic.php?f=9&t=7813

        // Code from original method
        /*bool collides = (proxy0->m_collisionFilterGroup & m_collisionFilterMask) != 0;
        collides = collides && (m_collisionFilterGroup & proxy0->m_collisionFilterMask);
        return collides;*/
        
        // Try to get pointer to the collision object
        btCollisionObject *obj = NULL;
        obj = static_cast<btCollisionObject*>(proxy0->m_clientObject);

        if (obj) {
            if (static_cast<int*>(obj->getUserPointer())) {
                if (static_cast<int*>(obj->getUserPointer())[0] == 10) {
                    // Accept ray collision
                    return true;
                }
            }
        }

        // Ignore ray collision
        return false;
    }
};

#endif  // TRACKRAYCALLBACK_H
